load("Dataset_GSAT.Rdata")
year = 1850:2100
year_obs = 1850:2019
ref_obs = 1961:1990
ny = length(year)
ny_obs = length(year_obs)
Nres = 100		# Number of resampling (bootstrap)

# Modelling IV in obs
message("Modelling IV")
raw_mmm		= apply(Z_fit[as.character(year),"all",],1,mean,na.rm=T)
raw_mmm_c	= raw_mmm	- mean(raw_mmm[year %in% ref_obs])
res			= Y[,"median"] - raw_mmm_c[as.character(year_obs)]
source("routines/outils.R")
source("routines/estim_mar2.R")
	message("   Fitting MAR process")
theta2_obs	= estim_mar2_link(res)
SY_mar2_obs = Sigma_mar2(theta2_obs,ny_obs)

# Variance in observations
Sigma_obs = SY_mar2_obs + var(t(Y[,-1]))

# Prior
message("Using models to construct a prior")
prior_mean = apply(Z_fit[1:251,"all",],1,mean)
prior_var = var(t(Z_fit[1:251,"all",]))



## Prepare to apply constraint (ie Kriging, Gaussian Conditioning, call it whatever you like)
message("Applying the constraint")
Center_X  = diag(rep(1, ny )) - rep(1,ny  )%o%(   year    %in% ref_obs)/length(ref_obs)
Center_Y  = diag(rep(1,ny_obs)) - rep(1,ny_obs)%o%(year_obs %in% ref_obs)/length(ref_obs)
H  = Center_X[year %in% year_obs,]		# Restriction to the observed period
y  = Center_Y %*% Y[,"median"]
SY = Center_Y %*% Sigma_obs %*% t(Center_Y)

# Apply constrain, ie derive posterior distribution
source("routines/generic_constrain.R")
post_dist = generic_constrain(prior_mean,prior_var,y,SY,H)


## Illustrative plot -- compute anomalies wrt ref_obs
message("Plot")
ref_plot = 1850:1900
Center_p = diag(rep(1, ny )) - rep(1,ny  )%o%(   year    %in% ref_plot)/length(ref_plot)
prior_anom_mean = Center_p%*%prior_mean
	names(prior_anom_mean) = as.character(year)
prior_anom_var = diag(Center_p %*% prior_var %*% t(Center_p))
	names(prior_anom_var) = as.character(year)
prior_anom_min = prior_anom_mean-1.64*sqrt(prior_anom_var)
	names(prior_anom_min) = as.character(year)
prior_anom_max = prior_anom_mean+1.64*sqrt(prior_anom_var)
	names(prior_anom_max) = as.character(year)
post_anom_mean = Center_p%*%post_dist$mean
	names(post_anom_mean) = as.character(year)
post_anom_var = diag(Center_p %*% post_dist$var %*% t(Center_p))
	names(post_anom_var) = as.character(year)
post_anom_min = post_anom_mean - 1.64*sqrt(post_anom_var)
	names(post_anom_min) = as.character(year)
post_anom_max = post_anom_mean + 1.64*sqrt(post_anom_var)
	names(post_anom_max) = as.character(year)
plot(year, prior_anom_mean, type="l", ylim=range(prior_anom_min,prior_anom_max), xlab="Year", ylab="Temperature change (C)",font=2,font.lab=2)
polygon(c(year,rev(year)), c(prior_anom_max,rev(prior_anom_min)),border=NA,col=rgb(1,0,0,alpha=.2))
polygon(c(year,rev(year)), c(post_anom_max,rev(post_anom_min)),border=NA,col=rgb(1,0,0,alpha=.5))
colb = col2rgb("brown")/255
lines(year,prior_anom_mean,lwd=1.5,col=rgb(colb[1],colb[2],colb[3],alpha=.5))
lines(year,post_anom_mean	,lwd=2,col=rgb(colb[1],colb[2],colb[3],alpha=1))
points(year_obs,Y[,"median"]-mean(Y[as.character(ref_plot),"median"]),pch=16, cex=.8)

message("Un-constrained SSP2-4.5 warming in 2100: ",#
		  format(prior_anom_mean["2100"],digits=1,nsmall=1),#
		  "°C +/- ",#
		  format(1.64*sqrt(prior_anom_var["2100"]),digits=1,nsmall=1),#
		  "°C")

message("Constrained SSP2-4.5 warming in 2100: ",#
		  format(post_anom_mean["2100"],digits=1,nsmall=1),#
		  "°C +/- ",#
		  format(1.64*sqrt(post_anom_var["2100"]),digits=1,nsmall=1),#
		  "°C")

